@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2007 Joerg Schulenburg
+Copyright (C) 2000-2009 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -353,7 +353,7 @@ wchar_t ocr_db(struct box *box1) {
         dist = dd;
         box3 = box2; /* box3 is a pointer and not copied box2 */
 
-        if (dist<100 && 100-dist > JOB->cfg.certainty) {
+        if (dist<100 && 100-dist >= JOB->cfg.certainty) {
           /* some deviation of the pattern is tolerated */
           int i, wa;
           for (i=0;i<box3->num_ac;i++) {
