@@ -67,7 +67,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       lines->x0[0] = p->x;  /* expand to left end during detection */
       lines->x1[0] = 0;	    /* expand to right end */
       lines->pitch[0] = JOB->cfg.spc; /* default word pitch */
-      lines->mono[0]  = 0;            /* default spacing = prop */
+      lines->mono[0]  = 0;            /* default spacing, 0 = prop */
       lines->num++;
   }
   i = lines->num;
@@ -362,7 +362,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       lines->m2[i] = m2;
       lines->m1[i] = m1;
       lines->pitch[i] = JOB->cfg.spc; /* default word pitch */
-      lines->pitch[i] = 0;            /* default spacing */
+      lines->mono[i] = 0;             /* default spacing, 0=prop, 1=mono */
       if (JOB->cfg.verbose & 16)
 	fprintf(stderr, " m= %4d %+3d %+3d %+3d w= %d (line=%d)\n#  ",
 		m1, m2 - m1, m3 - m1, m4 - m1, lines->wt[i], i);
