@@ -77,8 +77,10 @@ static void help(void) {
 	  " -v num    - verbose (see manual page)\n"
 	  " -c string - list of chars (debugging, see manual)\n"
 	  " -C string - char filter (ex. hexdigits: ""0-9A-Fx"", only ASCII)\n"
-	  " -m num    - operation modes (bitpattern, see manual)\n"
-	  " -a num      value of certainty (in percent, 0..100, default=95)\n");
+	  " -m num    - operation modes (bitpattern, see manual)\n");
+  fprintf(stderr, /* string length less than 509 bytes for ISO C89 */
+	  " -a num    - value of certainty (in percent, 0..100, default=95)\n"
+	  " -u string - output this string for every unrecognized character\n");
   fprintf(stderr, /* string length less than 509 bytes for ISO C89 */
 	  " examples:\n"
 	  "\tgocr -m 4 text1.pbm                   # do layout analyzis\n"
@@ -224,6 +226,10 @@ static void process_arguments(job_t *job, int argn, char *argv[])
 	job->cfg.certainty = atoi(s1);;
 	i++;
 	break;
+      case 'u': /* output marker for unrecognized chars */
+        job->cfg.unrec_marker = s1;
+        i++;
+        break;
       default:
 	fprintf(stderr, "# unknown option use -h for help\n");
       }
