@@ -1,7 +1,7 @@
 /* ocr-engine numbers only */
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2007 Joerg Schulenburg
+Copyright (C) 2000-2009 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -493,14 +493,16 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
        && box1->frame_vector[i7][1]<=y0+2*dy/3) {
         MSG(fprintf(stderr,"7-segment-type");)
       } else { /* regular-book-type */
-        if (aa[3][0]>=x1-dx/8
-         && aa[3][1]<=y0+dy/8) ad=99*ad/100;
-        if (aa[0][0]<=x0+dx/8
-         && aa[0][1]<=y0+dy/8) ad=99*ad/100;
-        if (aa[3][2]<=aa[1][2]) ad=97*ad/100;
+        MSG( fprintf(stderr,"upper bow not z-like? ad %d", ad); )
+        if (aa[3][0]>=x1-dx/8                // x of upper right point
+         && aa[3][1]< y0+dy/8) ad=99*ad/100; // y of upper right point
+        if (aa[0][0]<=x0+dx/8                // x of upper left point
+         && aa[0][1]< y0+dy/8) ad=99*ad/100; // y of upper left point
+        if (aa[3][2]<=aa[1][2]) ad=97*ad/100; // dist to (maxx,0) <= (0,maxy)
       }
       // ToDo: output no=(x,y)
-      MSG(fprintf(stderr,"i1-7 %d %d %d %d %d %d %d",i1,i2,i3,i4,i5,i6,i7);)
+      MSG( fprintf(stderr,"i1-7 %d %d %d %d %d %d %d ad %d",
+           i1,i2,i3,i4,i5,i6,i7,ad); )
       if (i5==i6) Break; // ~+
       
       if (box1->frame_vector[i5][1]
@@ -509,20 +511,27 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if (box1->frame_vector[i1][0]>x0+dx/8) Break;
       if (box1->frame_vector[i2][1]>(y0+  y1)/2) Break; 
       if (box1->frame_vector[i2][1]>(5*y0+3*y1)/8) ad=99*ad/100; 
+      MSG( fprintf(stderr,"ad %d", ad); )
       if (box1->frame_vector[i2][0]<(x0+x1+1)/2) Break;  // fat tiny fonts?
       if (box1->frame_vector[i2][0]<(x0+2*x1)/3) ad=99*ad/100;
+      MSG( fprintf(stderr,"ad %d", ad); )
       if (box1->frame_vector[i3][0]>(3*x0+x1)/4) Break;
       if (box1->frame_vector[i3][0]>(7*x0+x1)/8) ad=99*ad/100;
+      MSG( fprintf(stderr,"ad %d", ad); )
       if (box1->frame_vector[i3][1]<(y0+3*y1)/4) Break;
       if (box1->frame_vector[i3][1]>(y0+7*y1)/8) ad=99*ad/100;
+      MSG( fprintf(stderr,"ad %d", ad); )
       /* check lower leftmost point from right side */
       if (box1->frame_vector[i5][0]>(x0+2*x1)/3) Break;
       if (box1->frame_vector[i5][0]>(x0+  x1)/2) ad=98*ad/100;
       if (box1->frame_vector[i5][0]>(2*x0+x1)/3) ad=99*ad/100;
+      MSG( fprintf(stderr,"ad %d", ad); )
       if (box1->frame_vector[i5][1]<(3*y0+5*y1)/8) Break;
       if (box1->frame_vector[i5][1]<(y0+3*y1)/4) ad=99*ad/100;
+      MSG( fprintf(stderr,"ad %d", ad); )
       if (box1->frame_vector[i6][1]>(y0+2*y1)/3) Break;
       if (box1->frame_vector[i6][1]>(y0+  y1)/2) ad=99*ad/100;
+      MSG( fprintf(stderr,"ad %d", ad); )
       if (box1->frame_vector[i6][0]<(x0+3*x1)/4) Break;
       if (box1->frame_vector[i6][0]<(x0+7*x1)/8) ad=99*ad/100;
  
@@ -530,7 +539,7 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
      
       /* check if lower left and right points are joined directly */
       ld=line_deviation(box1, i3, i4);
-      MSG(fprintf(stderr," i1-i2 %d %d dist= %d/%d",i1,i2,ld,2*sq(1024/4));)
+      MSG(fprintf(stderr,"i1-i2 %d %d dist= %d/%d",i1,i2,ld,2*sq(1024/4));)
       if (ld >2*sq(1024/4)) Break;
       if (ld >  sq(1024/4)) ad=99*ad/100;
 
@@ -662,6 +671,8 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
         i6=nearest_frame_vector(box1, i5, i1, (x0+3*x1)/4, y1-dy/8);
         i7=nearest_frame_vector(box1, i5, i1,  x0        , y1-dy/8);
         MSG(fprintf(stderr,"open type");)
+        if (box1->frame_vector[i6][1]-y0>3*dy/4
+         || box1->frame_vector[i7][1]-y0>3*dy/4) ad=96*ad/100; // ~uU
       }
       // ToDo: output no=(x,y)
       MSG(fprintf(stderr,"i1-7 %d %d %d %d %d %d %d",i1,i2,i3,i4,i5,i6,i7);)
@@ -855,6 +866,13 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if (sdata->holes.num != 1) Break;
       if (sdata->holes.hole[0].y1 < dy/2) ad=95*ad/100; // whats good for?
       if (sdata->holes.hole[0].y0 < dy/4) Break;
+      MSG( fprintf(stderr,"hole[0].x0,x1 %d %d", sdata->holes.hole[0].x0,
+           sdata->holes.hole[0].x1); )
+      if (sdata->holes.hole[0].x0<1
+        && dx-1-sdata->holes.hole[0].x1>2) ad=ad*99/100; // melted serif sS ?
+      if (loop(box1->p,x0,y0+dy/2,dx,cs,0,RI)>0
+       && loop(box1->p,x0,y0+dy/4,dx,cs,0,RI)==0
+       && loop(box1->p,x0,y1-dy/4,dx,cs,0,RI)==0) ad=97*ad/100; // molten serif sS
 //      if( num_hole ( x0, x1, y0, y0+dy/2,box1->p,cs,NULL) >  0 ) ad=95*ad/100; 
 //      if( num_hole ( x0, x1, y0+dy/4, y1,box1->p,cs,NULL) != 1 ) Break; 
 //      if( num_hole ( x0, x1, y0     , y1,box1->p,cs,NULL) != 1 ) Break; 
@@ -1079,6 +1097,13 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
         y=loop(bp,dx-x-1,  dy-1,dy,cs,0,UP);
         if(y<dy/8) Break; // ~q (serif!)
       }
+      // check for agglutinated serif y
+      if (dy>=16 && dx>9
+        && loop(bp,   0,dy/ 4,dx,cs,0,RI)
+          -loop(bp,   0,dy/16,dx,cs,0,RI)>dx/6
+        && loop(bp,dx-1,dy/ 4,dx,cs,0,LE)
+          -loop(bp,dx-1,dy/16,dx,cs,0,LE)>dx/6) Break; // ~ serif yY
+          
       if (box1->m3) {
         if( gchar) ad=99*ad/100;  /* unsure */
         if(!hchar) ad=99*ad/100;  /* unsure */
@@ -1260,5 +1285,85 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       Setac(box1,(wchar_t)'0',ad);
       break;
    } 
+   // --- test 0 with a straight dot in it -------------------
+   for(ad=100;dx>4 && dy>5;){  /* v0.46+ */
+      DBG( char c_ask='0'; )
+      if (sdata->holes.num != 1) Break; /* do not be tolerant */
+      if (sdata->box1->num_frames != 2) ad=85*ad/100;
+      if( get_bw(x0      , x0+dx/2,y0+dy/2 , y0+dy/2,box1->p,cs,1) != 1 ) Break;
+      if( get_bw(x1-dx/2 , x1     ,y0+dy/2 , y0+dy/2,box1->p,cs,1) != 1 ) Break;
+      if( get_bw(x0+dx/2 , x0+dx/2,y1-dy/2 , y1,box1->p,cs,1) != 1 ) Break;
+      if( get_bw(x0+dx/2 , x0+dx/2,y0      , y0+dy/2,box1->p,cs,1) != 1 ) Break;
+      if( get_bw(x0+dx/2 , x0+dx/2,y0+dy/3 , y1-dy/3,box1->p,cs,1) != 1 ) Break;
+      // out_x(box1); printf(" x0 y0 %d %d\n",x0,y0);
+      if( num_cross(x0+dx/2,x0+dx/2,y0      , y1     ,box1->p,cs)  != 3 ) Break;
+      if( num_cross(x0+dx/3,x1-dx/3,y0      , y0     ,box1->p,cs)  != 1 ) // AND
+      if( num_cross(x0+dx/3,x1-dx/3,y0+1    , y0+1   ,box1->p,cs)  != 1 ) Break;
+      if( num_cross(x0+dx/3,x1-dx/3,y1      , y1     ,box1->p,cs)  != 1 ) // against "rauschen"
+      if( num_cross(x0+dx/3,x1-dx/3,y1-1    , y1-1   ,box1->p,cs)  != 1 ) Break;
+      if( num_cross(x0     ,x0     ,y0+dy/3 , y1-dy/3,box1->p,cs)  != 1 )
+      if( num_cross(x0+1   ,x0+1   ,y0+dy/3 , y1-dy/3,box1->p,cs)  != 1 ) Break;
+      if( num_cross(x1     ,x1     ,y0+dy/3 , y1-dy/3,box1->p,cs)  != 1 )
+      if( num_cross(x1-1   ,x1-1   ,y0+dy/3 , y1-dy/3,box1->p,cs)  != 1 ) Break;
+      // if( num_hole(x0,x1,y0,y1,box1->p,cs,NULL) != 2 ) Break;
+      
+      if( loop(bp,0   ,        0,x1-x0,cs,0,RI)<=
+          loop(bp,0   ,  2+dy/32,x1-x0,cs,0,RI)  ) Break;
+      x=  loop(bp,0   ,dy/2  ,x1-x0,cs,0,RI);
+      i=  loop(bp,0   ,dy/2-1,x1-x0,cs,0,RI); if (i>x) x=i;
+      i=  loop(bp,0   ,dy/2-2,x1-x0,cs,0,RI); if (i>x && dy>8) x=i;
+      if( loop(bp,0   ,  dy/4,x1-x0,cs,0,RI)<x ) Break; // ~8
+      x=  loop(bp,dx-1,dy/2  ,x1-x0,cs,0,LE);
+      i=  loop(bp,dx-1,dy/2-1,x1-x0,cs,0,LE); if(i>x) x=i;
+      i=  loop(bp,dx-1,dy/2-1,x1-x0,cs,0,LE); if(i>x && dy>8) x=i;
+      if( loop(bp,dx-1,3*dy/4,x1-x0,cs,0,LE)<x) Break; // ~8
+
+      x=loop(bp,dx-1,dy-1-dy/3,x1-x0,cs,0,LE);	// should be minimum
+      for( y=dy-1-dy/3;y<dy;y++ ){
+        i=loop(bp,dx-1,y,x1-x0,cs,0,LE);
+        if (i<x-dx/16) break; 
+        if (i>x) x=i;
+      }
+      if( y<dy ) Break;
+      
+      /* test for straight line */
+      y =loop(bp,dx/2,dy-1  ,y1-y0,cs,0,UP); if(y>dy/4) Break;
+      y+=loop(bp,dx/2,dy-1-y,y1-y0,cs,1,UP); if(y>dy/3) Break; if (y>dy/4) ad=ad*99/100;
+      y+=loop(bp,dx/2,dy-1-y,y1-y0,cs,0,UP); if(3*y>2*dy) Break;
+      x =loop(bp,dx/2,dy-y,dx/2,cs,0,RI);    if(x==0) Break;
+      // MM; fprintf(stderr," y=%d x=%d\n",y-1,x);
+      if( loop(bp,dx/2+x-1-dx/16,dy-y,y1-y0,cs,0,UP)==0 ) Break;
+       // $
+      for(i=0,y=dy/4;y<dy-dy/4-1;y++)
+      if( loop(bp,   0,y,dx-1,cs,0,RI) > dx/4
+       || loop(bp,dx-1,y,dx-1,cs,0,LE) > dx/4 ) break;
+      if( y<dy-dy/4-1 ) Break;
+
+      // ~D
+      if(     loop(bp,0,     dy/16,dx,cs,0,RI)
+         +    loop(bp,0,dy-1-dy/16,dx,cs,0,RI)
+         <= 2*loop(bp,0,     dy/2 ,dx,cs,0,RI)+dx/8 ) Break; // not konvex
+
+      if( loop(bp,dx-1,     dy/16,dx,cs,0,LE)>dx/8 )
+      if( loop(bp,0   ,     dy/16,dx,cs,0,RI)<dx/16 ) Break;
+      if( loop(bp,dx-1,dy-1-dy/16,dx,cs,0,LE)>dx/8 )
+      if( loop(bp,0   ,dy-1-dy/16,dx,cs,0,RI)<dx/16 ) Break;
+      if( get_bw(x1-dx/32,x1,y0,y0+dy/32,box1->p,cs,1) == 0
+       && get_bw(x1-dx/32,x1,y1-dy/32,y1,box1->p,cs,1) == 0
+       && ( get_bw(x0,x0+dx/32,y0,y0+dy/32,box1->p,cs,1) == 1
+         || get_bw(x0,x0+dx/32,y1-dy/32,y1,box1->p,cs,1) == 1 ) ) Break; // ~D
+
+      /* 5x9 font "9" is like "0" */
+      if (dx<16)
+      if ( num_cross(x0,x0,y0,y1,box1->p,cs)  != 1 ) ad=98*ad/100;
+
+       // italic a
+      for(i=0,y=6*dy/8;y<dy-dy/16;y++)
+      if( num_cross(0,dx-1,y,y,bp,cs) > 2 ) i++; else i--;
+      if(i>0) ad=ad*98/100; // ~'a' \it a
+      if( !hchar ) ad=90*ad/100;
+      Setac(box1,(wchar_t)'0',ad);
+      break;
+   } 
    return box1->c;
 }
