@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2008  Joerg Schulenburg
+Copyright (C) 2000-2009  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,6 +34,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 /* #include <unistd.h> */
 #endif
 
+/* Windows needs extra code to work fine, ^Z in BMP's will stop input else.
+ * I do not have any idea when this text mode will be an advantage
+ * but the MS community seems to like to do simple things in a complex way. */
+#if defined(O_BINARY) && (defined(__WIN32) || defined(__WIN32__)\
+ || defined(__WIN64) || defined(__WIN64__) || defined(__MSDOS__))
+# include <fcntl.h>
+# define SET_BINARY(_f) do {if (!isatty(_f)) setmode (_f, O_BINARY);} while (0)
+#else
+# define SET_BINARY(f) (void)0
+#endif
+
 #include "pnm.h"
 #ifdef HAVE_PAM_H
 # include <pam.h>
@@ -140,6 +151,7 @@ int readpgm(char *name, pix * p, int vvv) {
     /* open file; test if conversion is needed. */
     if (name[0] == '-' && name[1] == '\0') {
       fp = stdin;
+      SET_BINARY (fileno(fp)); // Windows needs it for correct work
     }
     else {
       pip = testsuffix(name);
@@ -157,7 +169,14 @@ int readpgm(char *name, pix * p, int vvv) {
 #ifdef HAVE_POPEN
         /* potential security vulnerability, if name contains tricks */
         /* example: gunzip -c dummy | rm -rf * */
+        /* windows needs "rb" for correct work, linux not, cygwin? */
+        /* ToDo: do you have better code to go arround this? */
+#if defined(__WIN32) || defined(__WIN32__) || defined(__WIN64) || defined(__WIN64__)
+	fp = popen(buf, "rb");  /* ToDo: may fail, please report */
+	if (!fp) fp = popen(buf, "r"); /* 2nd try, the gnu way */
+#else
         fp = popen(buf, "r");
+#endif
 #else
         F0("sorry, compile with HAVE_POPEN to use pipes");
 #endif
@@ -280,6 +299,7 @@ int readpgm( char *name, pix *p, int vvv){
     pip=NULL;
     if (name[0]=='-' && name[1]==0) {
       f1=stdin;  /* is this correct ??? */
+      SET_BINARY (fileno(f1)); // Windows needs it for correct work
     } else {
       pip=testsuffix(name);
       if (!pip) {
@@ -288,7 +308,12 @@ int readpgm( char *name, pix *p, int vvv){
         sprintf(buf,"%s \"%s\"",pip,name); /* ToDo: how to prevent OVL ? */
         if (vvv) { fprintf(stderr,"# popen( %s )\n",buf); }
 #ifdef HAVE_POPEN
+#if defined(__WIN32) || defined(__WIN32__) || defined(__WIN64) || defined(__WIN64__)
+	f1 = popen(buf, "rb");  /* ToDo: may fail, please report */
+	if (!f1) f1 = popen(buf, "r"); /* 2nd try, the gnu way */
+#else
         f1=popen(buf,"r");
+#endif
 #else
         F0("only PNM files supported (compiled without HAVE_POPEN)");
 #endif
@@ -443,13 +468,15 @@ int writeppm(char *nam, pix *p){ /* P6 raw-ppm */
   if (!f1) {
     strncpy(buf,"pnmtopng > ",12);  /* no spaces within filenames allowed! */
     strncpy(buf+11,nam,111); buf[123]=0;
-    strncpy(buf+strlen(buf),".png",5); 
+    strncpy(buf+strlen(buf),".png",5);
+    /* we dont care about win "wb" here, never debug on win systems */
     f1 = popen(buf, "w"); if(f1) f1t=1; else E0("popen pnmtopng");
   }
   if (!f1) {
     strncpy(buf,"gzip -c > ",11);
     strncpy(buf+10,nam,109); buf[120]=0;
     strncpy(buf+strlen(buf),".ppm.gz",8); 
+    /* we dont care about win "wb" here, never debug on win systems */
     f1 = popen(buf, "w"); if(f1) f1t=1; else E0("popen gzip -c");
   }
 #endif
